@@ -20,7 +20,7 @@
  *
  */
 
-/* $Id: easyh10_database.c,v 1.40 2005/10/20 17:13:09 nyaochi Exp $ */
+/* $Id: easyh10_database.c,v 1.41 2006/07/08 19:58:08 nyaochi Exp $ */
 
 #ifdef	HAVE_CONFIG_H
 #include <config.h>
@@ -745,12 +745,8 @@ int easyh10_database(
 	/* Change track number according to the user's demand. */
 	order_type = (flags & EASYH10_DATABASE_ORDERUNMASK);
 
-	/* We cannot use "default" ordering when right-to-left mode. */
-	if ((flags & EASYH10_DATABASE_RIGHTTOLEFT) && order_type == EASYH10_DATABASE_ORDER_LEAVE) {
-		order_type = EASYH10_DATABASE_ORDER_TRACKNUMBER;
-	}
-
-	if (order_type) {
+	/* Change the order of items only when: */
+	if (order_type || ((flags & EASYH10_DATABASE_RIGHTTOLEFT) && order_type == EASYH10_DATABASE_ORDER_LEAVE)) {
 		int num = ffd.num_targets;
 		sorting_track_t* array = (sorting_track_t*)malloc(sizeof(sorting_track_t) * ffd.num_targets);
 		if (!array) {
@@ -769,6 +765,13 @@ int easyh10_database(
 		}
 
 		switch (order_type) {
+		case EASYH10_DATABASE_ORDER_LEAVE:
+			/* Emulate the "default" ordering in RTL mode. */
+			if (flags & EASYH10_DATABASE_RIGHTTOLEFT) {
+				/* Order tracks in an alphabetical order of track titles. */
+				qsort(array, num, sizeof(sorting_track_t), compare_titlename);
+			}
+			break;
 		case EASYH10_DATABASE_ORDER_TRACKNAME:
 			qsort(array, num, sizeof(sorting_track_t), compare_titlename);
 			break;
@@ -809,6 +812,13 @@ int easyh10_database(
 				gettag_force_order(h10db, ffd.targets[j].entry, (uint32_t)i, ffd.num_targets);
 				h10db_set_tracknumber(h10db, ffd.targets[j].entry, tracknumber);
 			}
+		} else if ((flags & EASYH10_DATABASE_RIGHTTOLEFT) && order_type == EASYH10_DATABASE_ORDER_LEAVE) {
+			for (i = 0;i < ffd.num_targets;i++) {
+				int j = array[i].index;
+				uint32_t tracknumber = h10db_get_tracknumber(h10db, ffd.targets[j].entry);
+				gettag_force_order(h10db, ffd.targets[j].entry, (uint32_t)i, ffd.num_targets);
+				h10db_set_tracknumber(h10db, ffd.targets[j].entry, tracknumber);
+			}
 		} else {
 			for (i = 0;i < ffd.num_targets;i++) {
 				ffd.targets[array[i].index].info.tracknumer = (uint32_t)i;
