@@ -20,7 +20,7 @@
  *
  */
 
-/* $Id: easyh10_playlist.c,v 1.22 2005/10/21 20:16:18 nyaochi Exp $ */
+/* $Id: easyh10_playlist.c,v 1.27 2006/02/07 01:45:40 nyaochi Exp $ */
 
 #ifdef	HAVE_CONFIG_H
 #include <config.h>
@@ -35,6 +35,7 @@
 
 #include "easyh10.h"
 #include <playlist.h>
+#include <regex_playlist.h>
 #include <filepathutil.h>
 
 struct tag_playlist_target_file {
@@ -49,6 +50,7 @@ struct tag_findfile_playlist_dat {
 	playlist_target_file_t* playlists;
 	int num_playlists;
 	int playlist_in_music;
+	int regex_playlist;
 	void *instance;
 	update_progress_callback proc;
 };
@@ -71,6 +73,7 @@ typedef struct tag_findfile_music findfile_music_t;
 static const ucs2_char_t ucs2cs_m3u[] = {'.','m','3','u',0};
 static const ucs2_char_t ucs2cs_m3u8[] = {'.','m','3','u','8',0};
 static const ucs2_char_t ucs2cs_pls[] = {'.','p','l','s',0};
+static const ucs2_char_t ucs2cs_rep[] = {'.','r','e','p',0};
 
 
 #ifdef	_WIN32
@@ -86,6 +89,18 @@ static int comp_filename(const void *_x, const void *_y)
 	return COMP_STR(x->file, y->file);
 }
 
+static int comp_pathfile(const void *_x, const void *_y)
+{
+	media_target_file_t* x = (media_target_file_t*)_x;
+	media_target_file_t* y = (media_target_file_t*)_y;
+	int ret = COMP_STR(x->path, y->path);
+	if (ret == 0) {
+		return COMP_STR(x->file, y->file);
+	} else {
+		return ret;
+	}
+}
+
 static int find_musicfile(findfile_music_t* ffd_music, const ucs2_char_t* filename, int *begin, int *end)
 {
 	int low = 0, high = ffd_music->num_targets-1;
@@ -371,7 +386,7 @@ void generate_plp_filename(ucs2_char_t* dst, const ucs2_char_t* extension, const
 		ucs2_char_t filename[MAX_PATH];
 		ucs2cpy(filename, src);
 		filepathutil_remove_extension(filename);
-		filepathutil_add_extension(filename, extension);
+		ucs2cat(filename, extension);
 		ucs2cpy(dst, filename);
 	}
 }
@@ -382,7 +397,8 @@ static int found_file_playlist(void *instance, const ucs2_char_t* found_path, co
 
 	if (filepathutil_hasext(found_file, ucs2cs_m3u) || 
 		filepathutil_hasext(found_file, ucs2cs_m3u8) ||
-		filepathutil_hasext(found_file, ucs2cs_pls)
+		filepathutil_hasext(found_file, ucs2cs_pls) ||
+		(ffd->regex_playlist && filepathutil_hasext(found_file, ucs2cs_rep))
 		) {
 		/* Playlist (M3U/M3U8/PLS) file. */
 		playlist_target_file_t* new_target = NULL;
@@ -461,8 +477,84 @@ static int found_file_music(void *instance, const ucs2_char_t* found_path, const
 	return 0;
 }
 
+static void copy_music_list(findfile_music_t* dst, const findfile_music_t* src)
+{
+	int i;
 
+	memset(dst, 0, sizeof(*dst));
+	dst->num_targets = src->num_targets;
+	dst->targets = calloc(dst->num_targets, sizeof(media_target_file_t));
 
+	for (i = 0;i < dst->num_targets;++i) {
+		ucs2cpy(dst->targets[i].path, src->targets[i].path);
+		ucs2cpy(dst->targets[i].file, src->targets[i].file);
+	}
+}
+
+static int generate_regex_playlist(playlist_t* pl, const ucs2_char_t *filename, findfile_music_t* musics)
+{
+	int i = 0;
+	regex_playlist_t* rep = NULL;
+
+	/* Initialize an empty playlist. */
+	playlist_init(pl);
+
+	/* Initialize a regex playlist engine. */
+	regex_playlist_init(&rep);
+	if (!rep) {
+		return -1;
+	}
+
+	/* Read a regex playlist file. */
+	if (regex_playlist_read(rep, filename) != 0) {
+		return -1;
+	}
+	
+	/* Search for music files that matches the condition. */
+	for (i = 0;i < musics->num_targets;++i) {
+		ucs2_char_t str[MAX_PATH];
+		ucs2cpy(str, musics->targets[i].path);
+		ucs2cat(str, musics->targets[i].file);
+		filepathutil_decode(str);
+		if (regex_playlist_test(rep, str)) {
+			playlist_add(pl, str);
+		}
+	}
+	
+
+	/* Free the regex playlist engine. */
+	regex_playlist_finish(rep);
+	
+	return 0;
+}
+
+static const ucs2_char_t* check_playlist_for_shuffle(const ucs2_char_t* filename)
+{
+	static const ucs2_char_t ucs2cs_sfl[] = {'.','s','f','l'};
+	const ucs2_char_t *p = NULL, *q = NULL, *r = NULL;
+
+	q = filename + ucs2len(filename);
+	while (filename <= q) {
+		if (*q == '.') {
+			break;
+		}
+		--q;
+	}
+	if (q < filename) {
+		return NULL;
+	}
+	for (p = filename;p < q;++p) {
+		if (*p == '.') {
+			r = p;
+		}
+	}
+
+	if (r && ucs2ncmp(r, ucs2cs_sfl, 4) == 0) {
+		return r;
+	}
+
+	return NULL;
+}
 
 static int dummy_progress_callback(void *instance, int phase, int param_int, void *param_ptr)
 {
@@ -487,7 +579,7 @@ int easyh10_playlist(
 {
 	int i, ret, num_to_update, num_updates, num_success;
 	findfile_playlist_dat_t ffd;
-	findfile_music_t ffd_music;
+	findfile_music_t ffd_music, ffd_music_for_regex;
 	ucs2_char_t *p = NULL;
 
 	/* If no target specified, exit. */
@@ -505,11 +597,14 @@ int easyh10_playlist(
 		error_proc = dummy_error_callback;
 	}
 
+	memset(&ffd_music_for_regex, 0, sizeof(ffd_music_for_regex));
+
 	/* For progress report during enumeration. */
 	ffd.num_playlists = 0;
 	ffd.playlists = NULL;
 	ffd.instance = instance;
 	ffd.proc = progress_proc;
+	ffd.regex_playlist = (flags & EASYH10_PLAYLIST_REGEX);
 
 	/* Notify the start of playlist enumeration. */
 	progress_proc(instance, EASYH10P_ENUMTARGETS_START, -1, NULL);
@@ -534,12 +629,23 @@ int easyh10_playlist(
 	ffd_music.instance = instance;
 	ffd_music.proc = progress_proc;
 
-	if (flags & EASYH10_PLAYLIST_SEARCH_MISSING) {
+	if ((flags & EASYH10_PLAYLIST_SEARCH_MISSING) || (flags & EASYH10_PLAYLIST_REGEX)) {
 		progress_proc(instance, EASYH10P_PLAYLIST_ENUMMUSIC_START, -1, NULL);
 		find_file(path_to_music, 1, found_file_music, &ffd_music);
 		if (ffd_music.targets) {
-			/* We need to sort music files in an alphabetical order of file names for binary search. */
-			qsort(ffd_music.targets, ffd_music.num_targets, sizeof(ffd_music.targets[0]), comp_filename);
+			if (flags & EASYH10_PLAYLIST_REGEX) {
+				/* We need to sort music files in an alphabetical order of path/file names. */
+				copy_music_list(&ffd_music_for_regex, &ffd_music);
+			}
+			if (flags & EASYH10_PLAYLIST_SEARCH_MISSING) {
+				/* We need to sort music files in an alphabetical order of file names for binary search. */
+				qsort(
+					ffd_music.targets,
+					ffd_music.num_targets,
+					sizeof(ffd_music.targets[0]),
+					comp_filename
+					);
+			}
 		}
 		progress_proc(instance, EASYH10P_PLAYLIST_ENUMMUSIC_END, ffd_music.num_targets, NULL);
 	}
@@ -550,18 +656,31 @@ int easyh10_playlist(
 		for (i = 0;i < ffd.num_playlists;i++) {
 			ucs2_char_t input[MAX_PATH], output[MAX_PATH];
 			playlist_target_file_t* target = &ffd.playlists[i];
+			int reconvert = 0;
 
 			/* Generate the input filename. */
 			filepathutil_combinepath(input, MAX_PATH, target->path, target->file);
 
-			/* Generate the output filename. */
-			generate_plp_filename(output, playlist_extension, input, path_to_playlist, target->playlist_in_music);
-
-			if (filepathutil_file_exists(output) && filepathutil_compare_lastupdate(input, output) <= 0) {
-				target->update = 0;
-			} else {
-				num_to_update++;
+			/* Check if this playlist will be shuffled. */
+			if (flags & EASYH10_PLAYLIST_AUTO_SHUFFLE) {
+				reconvert |= (check_playlist_for_shuffle(input) ? 1 : 0);
 			}
+
+			/* Check if this playlist is regex playlist. */
+			if ((flags & EASYH10_PLAYLIST_REGEX) && filepathutil_hasext(input, ucs2cs_rep)) {
+				reconvert |= 1;
+			}
+
+			/* Make sure to convert shuffling playlist every time. */
+			if (!reconvert) {
+				/* Generate the output filename. */
+				generate_plp_filename(output, playlist_extension, input, path_to_playlist, target->playlist_in_music);
+				if (filepathutil_file_exists(output) && filepathutil_compare_lastupdate(input, output) <= 0) {
+					target->update = 0;
+					continue;
+				}
+			}
+			num_to_update++;
 		}
 	} else {
 		num_to_update = ffd.num_playlists;
@@ -579,17 +698,14 @@ int easyh10_playlist(
 		playlist_target_file_t* target = &ffd.playlists[i];
 
 		if (target->update) {
-			/* Generate the input filename. */
-			filepathutil_combinepath(input, MAX_PATH, target->path, target->file);
-
-			/* Generate the output filename. */
-			generate_plp_filename(output, playlist_extension, input, path_to_playlist, target->playlist_in_music);
-
 			progress_proc(instance, EASYH10P_PLAYLIST, ++num_updates, (void*)(const ucs2_char_t*)target->file);
 
 			/* Initialize the playlist structure. */
 			playlist_init(&pl);
 
+			/* Generate the input filename. */
+			filepathutil_combinepath(input, MAX_PATH, target->path, target->file);
+
 			/* Read the playlist. */
 			if (filepathutil_hasext(target->file, ucs2cs_m3u)) {
 				if (playlist_m3u_read(&pl, input) != 0) {
@@ -606,8 +722,13 @@ int easyh10_playlist(
 					error_proc(instance, EASYH10_WARNING_PLAYLIST_READ, input);
 					continue;
 				}
+			} else if ((flags & EASYH10_PLAYLIST_REGEX) && filepathutil_hasext(target->file, ucs2cs_rep)) {
+				if (generate_regex_playlist(&pl, input, &ffd_music_for_regex) != 0) {
+					error_proc(instance, EASYH10_WARNING_PLAYLIST_READ, input);
+					continue;
+				}
 			}
-
+			
 			/* Normalize the content of the playlist. */
 			ret = normalize_playlist(
 				&pl,
@@ -621,11 +742,26 @@ int easyh10_playlist(
 				error_proc
 				);
 			if (ret == pl.num_entries || (ret > 0 && (flags & EASYH10_PLAYLIST_SKIP_MISSING))) {
+				const ucs2_char_t* shuffle = NULL;
+				
+				if (flags & EASYH10_PLAYLIST_AUTO_SHUFFLE) {
+					shuffle = check_playlist_for_shuffle(input);
+				}
+
+				/* Generate the output filename. */
+				generate_plp_filename(output, playlist_extension, input, path_to_playlist, target->playlist_in_music);
+
+				if (shuffle) {
+					playlist_shuffle(&pl);
+				}
+
 				/* Write the playlist. */
-				if (playlist_plp_write(&pl, output) != 0) {
-					error_proc(instance, EASYH10_WARNING_PLAYLIST_WRITE, output);
-				} else {
-					num_success++;
+				if (pl.num_entries > 0) {
+					if (playlist_plp_write(&pl, output) != 0) {
+						error_proc(instance, EASYH10_WARNING_PLAYLIST_WRITE, output);
+					} else {
+						num_success++;
+					}
 				}
 			}
 
@@ -636,6 +772,7 @@ int easyh10_playlist(
 	/* Notify the end of playlist conversion. */
 	progress_proc(instance, EASYH10P_PLAYLIST_END, num_success, NULL);
 
+	free(ffd_music_for_regex.targets);
 	free(ffd_music.targets);
 	free(ffd.playlists);
 	return 0;
