@@ -20,7 +20,7 @@
  *
  */
 
-/* $Id: main.c,v 1.90 2006/05/02 04:56:12 nyaochi Exp $ */
+/* $Id: main.c,v 1.94 2006/10/08 19:29:18 nyaochi Exp $ */
 
 #ifdef	HAVE_CONFIG_H
 #include <config.h>
@@ -41,6 +41,10 @@
 #include <filepathutil.h>
 #include <easyh10.h>
 
+#ifdef HAVE_LANGINFO_CODESET
+#include <langinfo.h>
+#endif
+
 #ifdef	HAVE_GETOPT_H
 #include <getopt.h>
 #else
@@ -48,11 +52,11 @@
 #endif
 
 #define	APPLICATION_S		"EasyH10 [CUI]"
-#define	VERSION_S			"1.4"
+#define	VERSION_S			"1.5"
 #define	COPYRIGHT_S			"Copyright (c) 2005-2006 by Nyaochi"
 
 #ifndef		_WIN32
-#define	SYSTEM_ENCODING		"UTF-8"
+#define	SYSTEM_ENCODING		"ISO-8859-1"
 #define	MUSICTAG_ENCODING	"ISO-8859-1"
 #endif
 
@@ -82,11 +86,8 @@ void usage(FILE *fp, const char *argv0)
 	fprintf(fp, "\n");
 
 	fprintf(fp, "Database construction options:\n");
-	fprintf(fp, "  -C, --construct       Construct a media database from music files\n");
-	fprintf(fp, "  -i, --incremental     Enable incremental update mode\n");
-	fprintf(fp, "  -c, --clean           Clean unused database entries in the existing database\n");
-	fprintf(fp, "  -U, --update          Update a media database (alias of options \"-Cic\")\n");
-	fprintf(fp, "  -n, --new             Discard the existing database and rebuild a new one\n");
+	fprintf(fp, "  -C, --construct       Construct a new media database from music files\n");
+	fprintf(fp, "  -U, --update          Update the existing media database incrementally\n");
 	fprintf(fp, "  -o, --order=TYPE      Specify a playback order:\n");
 	fprintf(fp, "          d, default        Leave an H10 firmware for playback order (DEFAULT)\n");
 	fprintf(fp, "          t, tracktitle     Force an alphabetical order of track titles\n");
@@ -645,6 +646,9 @@ int main(int argc, char *argv[])
 	const char *media_info_source = NULL;
 	FILE *fpi = stdin, *fpo = stdout, *fpe = stderr;
 	progress_data_t pd;
+#ifndef	_WIN32
+	int encoding_specified = 0;
+#endif/*_WIN32*/
 
 	pd.fp = fpe;
 	pd.num = 0;
@@ -681,9 +685,6 @@ int main(int argc, char *argv[])
 		int option_index = 0;
 		static const struct option long_options[] = {
 			{"construct",		no_argument,		0, 'C'},
-			{"new",				no_argument,		0, 'n'},
-			{"incremental",		no_argument,		0, 'i'},
-			{"clean",			no_argument,		0, 'c'},
 			{"update",			no_argument,		0, 'U'},
 			{"order",			required_argument,	0, 'o'},
 			{"source",			required_argument,	0, 'z'},
@@ -717,9 +718,9 @@ int main(int argc, char *argv[])
 			{NULL, 0, 0, 0}
 		};
 #ifndef	_WIN32
-		int c = getopt_long(argc, argv, "CnicUo:z:PMrgfkxlI:E:at:T:Xs:d:m:p:e:w:bDVhv", long_options, &option_index);
+		int c = getopt_long(argc, argv, "CUo:z:PMrgfkxlI:E:at:T:Xs:d:m:p:e:w:bDVhv", long_options, &option_index);
 #else
-		int c = getopt_long(argc, argv, "CnicUo:z:PMrgfkxlI:E:at:T:Xs:d:m:p:bDVhv", long_options, &option_index);
+		int c = getopt_long(argc, argv, "CUo:z:PMrgfkxlI:E:at:T:Xs:d:m:p:bDVhv", long_options, &option_index);
 #endif/*_WIN32*/
 		if (c == -1) {
 			break;
@@ -728,15 +729,7 @@ int main(int argc, char *argv[])
 		switch (c) {
 		case 'C':
 			mode |= MODE_DATABASE;
-			break;
-		case 'n':
-			easyh10_flag |= EASYH10_DATABASE_NEW;
-			break;
-		case 'i':
-			easyh10_flag |= EASYH10_DATABASE_INCREMENTAL;
-			break;
-		case 'c':
-			easyh10_flag |= EASYH10_DATABASE_CLEAN;
+			easyh10_flag |= (EASYH10_DATABASE_INCREMENTAL | EASYH10_DATABASE_CLEAN | EASYH10_DATABASE_NEW);
 			break;
 		case 'U':
 			mode |= MODE_DATABASE;
@@ -852,15 +845,18 @@ int main(int argc, char *argv[])
 			mbstoucs2(path_to_playlist, MAX_PATH, optarg, strlen(optarg)+1);
 			break;
 
-#ifndef		_WIN32
+#ifndef	_WIN32
 		case 'e':
 			if (set_encoding(optarg) != 0) {
+				fprintf(fpe, "ERROR: Initialization failed to convert %s <-> UCS-2LE.\n", optarg);
 				ret = 1;
 				goto exit_main;
 			}
+			encoding_specified = 1;
 			break;
 		case 'w':
 			if (set_encoding_music(optarg) != 0) {
+				fprintf(fpe, "ERROR: Initialization failed to convert %s <-> UCS-2LE.\n", optarg);
 				ret = 1;
 				goto exit_main;
 			}
@@ -888,6 +884,35 @@ int main(int argc, char *argv[])
 		}
 	}
 
+#ifndef	_WIN32
+	/* If no system encoding was specified, try to detect it. */
+	if (!encoding_specified) {
+		const char *encoding = getenv("CHARSET");
+
+#ifdef	HAVE_LANGINFO_CODESET
+		if (!encoding) {
+			encoding = nl_langinfo(CODESET);
+		}
+#endif/*HAVE_LANGINFO_CODESET*/
+
+		if (encoding) {
+			/* EasyH10 could detect the character encoding. */
+			if (set_encoding(encoding) != 0) {
+				fprintf(fpe, "ERROR: Initialization failed to convert %s <-> UCS-2LE.\n", encoding);
+				fprintf(fpe, "       Specify a character encoding manually by using -e option.\n");
+				ret = 1;
+				goto exit_main;
+			}
+		} else {
+			/* EasyH10 could not detect the character encoding. */
+			fprintf(fpe, "ERROR: Failed to detect the default character encoding of the current OS.\n");
+			fprintf(fpe, "       Specify the character encoding manually by using -e option.\n");
+			ret = 1;
+			goto exit_main;
+		}
+	}
+#endif/*_WIN32*/
+
 	/* Exit when version mode specified. */
 	if (mode & MODE_VERSION) {
 		goto exit_main;
