@@ -20,12 +20,13 @@
  *
  */
 
-/* $Id: ucs2char_iconv.c,v 1.26 2006/04/30 17:23:14 nyaochi Exp $ */
+/* $Id: ucs2char_iconv.c,v 1.27 2006/07/27 01:42:35 nyaochi Exp $ */
 
 #ifdef	HAVE_CONFIG_H
 #include <config.h>
 #endif/*HAVE_CONFIG_H*/
 
+#include <errno.h>
 #include <os.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -174,7 +175,7 @@ size_t iconv_convert(iconv_t cd, char **outbuf, size_t out_size, char **inbuf, s
 			char *p = buffer;
 			size_t outbytesleft = 1024;
 			int iconvret = iconv(cd, (const char **)inbuf, &inbytesleft, &p, &outbytesleft);
-			if (iconvret == -1) {
+			if (iconvret == -1 && errno != E2BIG) {
 				return 0;
 			}
 			ret += (1024 - outbytesleft);
